@@ -1,11 +1,11 @@
 use std::iter;
 
-use swc_atoms::{js_word, JsWord};
+use swc_atoms::JsWord;
 use swc_common::{util::take::Take, Mark, DUMMY_SP};
 use swc_ecma_ast::*;
 use swc_ecma_transforms_base::helper;
 use swc_ecma_transforms_classes::{get_prototype_of, visit_mut_only_key};
-use swc_ecma_utils::{private_ident, quote_ident, ExprFactory};
+use swc_ecma_utils::{quote_ident, ExprFactory};
 use swc_ecma_visit::{
     noop_visit_mut_type, noop_visit_type, Visit, VisitMut, VisitMutWith, VisitWith,
 };
@@ -44,10 +44,6 @@ impl SuperCallFinder {
 macro_rules! ignore_return {
     ($name:ident, $T:ty) => {
         fn $name(&mut self, n: &mut $T) {
-            if self.in_injected_define_property_call {
-                return;
-            }
-
             let old = self.ignore_return;
             self.ignore_return = true;
             n.visit_mut_children_with(self);
@@ -156,11 +152,6 @@ pub(super) fn constructor_fn(c: Constructor) -> Function {
 pub(super) struct ConstructorFolder<'a> {
     pub class_name: &'a Ident,
     pub mode: Option<SuperFoldingMode>,
-    /// Variables named `thisSuper` will be added if `super.foo` or
-    /// `super.foo()` is used in constructor.
-    pub vars: &'a mut Vec<VarDeclarator>,
-
-    pub cur_this_super: Option<Ident>,
 
     /// Mark for `_this`
     pub mark: Mark,
@@ -168,132 +159,6 @@ pub(super) struct ConstructorFolder<'a> {
     pub super_var: Option<Ident>,
     /// True when recursing into other function or class.
     pub ignore_return: bool,
-    pub in_injected_define_property_call: bool,
-}
-
-impl ConstructorFolder<'_> {
-    /// `true` means it's processed and children are folded.
-    fn handle_super_assignment(&mut self, e: &mut Expr) -> bool {
-        if let Expr::Assign(AssignExpr {
-            left,
-            op: op!("="),
-            right,
-            span,
-        }) = e
-        {
-            let left = left.take().normalize_expr();
-            match left {
-                PatOrExpr::Expr(mut left_expr) => match *left_expr {
-                    Expr::SuperProp(_) => {
-                        right.visit_mut_children_with(self);
-                        self.handle_super_access(&mut left_expr, Some(right.take()));
-                        *e = *left_expr;
-                        true
-                    }
-                    _ => {
-                        *e = Expr::Assign(AssignExpr {
-                            span: *span,
-                            left: PatOrExpr::Expr(left_expr),
-                            op: op!("="),
-                            right: right.take(),
-                        });
-                        false
-                    }
-                },
-                _ => {
-                    *e = Expr::Assign(AssignExpr {
-                        span: *span,
-                        left,
-                        op: op!("="),
-                        right: right.take(),
-                    });
-                    false
-                }
-            }
-        } else {
-            false
-        }
-    }
-
-    fn handle_super_access(&mut self, e: &mut Expr, set_to: Option<Box<Expr>>) {
-        if let Expr::SuperProp(SuperPropExpr { span, prop, .. }) = e {
-            let this_var = quote_ident!(DUMMY_SP.apply_mark(self.mark), "_this");
-            let this_super = private_ident!("_thisSuper");
-            self.cur_this_super = Some(this_super.clone());
-
-            self.vars.push(VarDeclarator {
-                span: DUMMY_SP,
-                name: this_super.clone().into(),
-                init: None,
-                definite: Default::default(),
-            });
-
-            // _thisSuper = _assertThisInitialized(_this)
-            let init_this_super = Box::new(Expr::Assign(AssignExpr {
-                span: DUMMY_SP,
-                op: op!("="),
-                left: PatOrExpr::Pat(Box::new(this_super.clone().into())),
-                right: Box::new(Expr::Call(CallExpr {
-                    span: DUMMY_SP,
-                    callee: helper!(assert_this_initialized, "assertThisInitialized"),
-                    args: vec![this_var.as_arg()],
-                    type_args: Default::default(),
-                })),
-            }));
-            // _getPrototypeOf(Foo.prototype)
-            let get_proto = Box::new(Expr::Call(CallExpr {
-                span: *span,
-                callee: helper!(get_prototype_of, "getPrototypeOf"),
-                args: vec![self
-                    .class_name
-                    .clone()
-                    .make_member(quote_ident!("prototype"))
-                    .as_arg()],
-                type_args: Default::default(),
-            }));
-
-            *e = Expr::Call(CallExpr {
-                span: DUMMY_SP,
-                callee: if set_to.is_some() {
-                    helper!(set, "set")
-                } else {
-                    helper!(get, "get")
-                },
-                args: {
-                    let mut args = vec![
-                        Expr::Seq(SeqExpr {
-                            span: DUMMY_SP,
-                            exprs: vec![init_this_super, get_proto],
-                        })
-                        .as_arg(),
-                        match prop {
-                            SuperProp::Computed(ComputedPropName { expr, .. }) => {
-                                expr.take().as_arg()
-                            }
-                            SuperProp::Ident(ident) => Str {
-                                span: ident.span,
-                                value: ident.take().sym,
-                                has_escape: false,
-                                kind: Default::default(),
-                            }
-                            .as_arg(),
-                        },
-                    ];
-
-                    let is_set = set_to.is_some();
-                    args.extend(set_to.map(|v| v.as_arg()));
-
-                    args.push(this_super.as_arg());
-
-                    if is_set {
-                        args.push(true.as_arg());
-                    }
-                    args
-                },
-                type_args: Default::default(),
-            })
-        };
-    }
 }
 
 /// `None`: `return _possibleConstructorReturn`
@@ -310,125 +175,66 @@ impl VisitMut for ConstructorFolder<'_> {
 
     visit_mut_only_key!();
 
-    ignore_return!(visit_mut_function, Function);
-
     ignore_return!(visit_mut_class, Class);
 
     ignore_return!(visit_mut_arrow_expr, ArrowExpr);
 
     ignore_return!(visit_mut_constructor, Constructor);
 
-    fn visit_mut_call_expr(&mut self, e: &mut CallExpr) {
-        match &e.callee {
-            Callee::Expr(callee) => match &**callee {
-                Expr::SuperProp(_) => {
-                    let old = self.cur_this_super.take();
-
-                    e.visit_mut_children_with(self);
-
-                    let this_super = self.cur_this_super.take().unwrap();
-                    self.cur_this_super = old;
-
-                    e.args.insert(0, this_super.as_arg());
-                    e.callee = e
-                        .callee
-                        .take()
-                        .expect_expr()
-                        .make_member(quote_ident!("call"))
-                        .as_callee();
-                }
-
-                _ => e.visit_mut_children_with(self),
-            },
-
-            _ => e.visit_mut_children_with(self),
-        }
-    }
+    fn visit_mut_function(&mut self, _: &mut Function) {}
 
     fn visit_mut_expr(&mut self, expr: &mut Expr) {
         match self.mode {
             Some(SuperFoldingMode::Assign) => {}
             _ => {
-                if !self.handle_super_assignment(expr) {
-                    expr.visit_mut_children_with(self);
-                };
-                self.handle_super_access(expr, None);
+                expr.visit_mut_children_with(self);
                 return;
             }
         }
 
-        // We pretend method folding mode for while folding injected `_defineProperty`
-        // calls.
+        expr.visit_mut_children_with(self);
+
         if let Expr::Call(CallExpr {
-            callee: Callee::Expr(callee),
+            callee: Callee::Super(..),
+            args,
             ..
         }) = expr
         {
-            if let Expr::Ident(Ident {
-                sym: js_word!("_defineProperty"),
-                ..
-            }) = &**callee
-            {
-                let old = self.in_injected_define_property_call;
-                self.in_injected_define_property_call = true;
-                expr.visit_mut_children_with(self);
-                self.in_injected_define_property_call = old;
-                return;
-            }
-        }
-
-        if !self.handle_super_assignment(expr) {
-            expr.visit_mut_children_with(self);
-        };
-        match expr {
-            Expr::This(e) => {
-                *expr = Expr::Ident(Ident::new("_this".into(), e.span.apply_mark(self.mark)))
-            }
-            Expr::Call(CallExpr {
-                callee: Callee::Super(..),
-                args,
-                ..
-            }) => {
-                let right = match self.super_var.clone() {
-                    Some(super_var) => Box::new(Expr::Call(CallExpr {
-                        span: DUMMY_SP,
-                        callee: if self.is_constructor_default {
-                            super_var.make_member(quote_ident!("apply")).as_callee()
-                        } else {
-                            super_var.make_member(quote_ident!("call")).as_callee()
-                        },
-                        args: if self.is_constructor_default {
-                            vec![
-                                ThisExpr { span: DUMMY_SP }.as_arg(),
-                                quote_ident!("arguments").as_arg(),
-                            ]
-                        } else {
-                            let mut call_args = vec![ThisExpr { span: DUMMY_SP }.as_arg()];
-                            call_args.extend(args.take());
-
-                            call_args
-                        },
-                        type_args: Default::default(),
-                    })),
-
-                    None => Box::new(make_possible_return_value(ReturningMode::Prototype {
-                        class_name: self.class_name.clone(),
-                        args: Some(args.take()),
-                        is_constructor_default: self.is_constructor_default,
-                    })),
-                };
-
-                *expr = Expr::Assign(AssignExpr {
+            let right = match self.super_var.clone() {
+                Some(super_var) => Box::new(Expr::Call(CallExpr {
                     span: DUMMY_SP,
-                    left: PatOrExpr::Pat(
-                        quote_ident!(DUMMY_SP.apply_mark(self.mark), "_this").into(),
-                    ),
-                    op: op!("="),
-                    right,
-                });
-            }
+                    callee: if self.is_constructor_default {
+                        super_var.make_member(quote_ident!("apply")).as_callee()
+                    } else {
+                        super_var.make_member(quote_ident!("call")).as_callee()
+                    },
+                    args: if self.is_constructor_default {
+                        vec![
+                            ThisExpr { span: DUMMY_SP }.as_arg(),
+                            quote_ident!("arguments").as_arg(),
+                        ]
+                    } else {
+                        let mut call_args = vec![ThisExpr { span: DUMMY_SP }.as_arg()];
+                        call_args.extend(args.take());
 
-            _ => self.handle_super_access(expr, None),
+                        call_args
+                    },
+                    type_args: Default::default(),
+                })),
+
+                None => Box::new(make_possible_return_value(ReturningMode::Prototype {
+                    class_name: self.class_name.clone(),
+                    args: Some(args.take()),
+                    is_constructor_default: self.is_constructor_default,
+                })),
+            };
+
+            *expr = Expr::Assign(AssignExpr {
+                span: DUMMY_SP,
+                left: PatOrExpr::Pat(quote_ident!(DUMMY_SP.apply_mark(self.mark), "_this").into()),
+                op: op!("="),
+                right,
+            });
         };
     }
 
@@ -631,35 +437,17 @@ pub(super) fn replace_this_in_constructor(mark: Mark, c: &mut Constructor) -> bo
         mark: Mark,
         found: bool,
         wrap_with_assertion: bool,
-        in_injected_define_property_call: bool,
     }
 
     impl VisitMut for Replacer {
         noop_visit_mut_type!();
 
-        fn visit_mut_class(&mut self, _: &mut Class) {}
+        // let computed keys be visited
+        fn visit_mut_constructor(&mut self, _: &mut Constructor) {}
+
+        fn visit_mut_function(&mut self, _: &mut Function) {}
 
         fn visit_mut_expr(&mut self, expr: &mut Expr) {
-            // We pretend method folding mode for while folding injected `_defineProperty`
-            // calls.
-            if let Expr::Call(CallExpr {
-                callee: Callee::Expr(callee),
-                ..
-            }) = expr
-            {
-                if let Expr::Ident(Ident {
-                    sym: js_word!("_defineProperty"),
-                    ..
-                }) = &**callee
-                {
-                    let old = self.in_injected_define_property_call;
-                    self.in_injected_define_property_call = true;
-                    expr.visit_mut_children_with(self);
-                    self.in_injected_define_property_call = old;
-                    return;
-                }
-            }
-
             match expr {
                 Expr::This(..) => {
                     self.found = true;
@@ -681,13 +469,6 @@ pub(super) fn replace_this_in_constructor(mark: Mark, c: &mut Constructor) -> bo
             }
         }
 
-        fn visit_mut_function(&mut self, n: &mut Function) {
-            if self.in_injected_define_property_call {
-                return;
-            }
-            n.visit_mut_children_with(self)
-        }
-
         fn visit_mut_member_expr(&mut self, expr: &mut MemberExpr) {
             if self.mark != Mark::root() {
                 let old = self.wrap_with_assertion;
@@ -704,9 +485,8 @@ pub(super) fn replace_this_in_constructor(mark: Mark, c: &mut Constructor) -> bo
         found: false,
         mark,
         wrap_with_assertion: true,
-        in_injected_define_property_call: false,
     };
-    c.visit_mut_with(&mut v);
+    c.visit_mut_children_with(&mut v);
 
     v.found
 }
