@@ -1191,7 +1191,8 @@ fn can_be_null(e: &Expr) -> bool {
         Expr::TsNonNull(..) => false,
         Expr::TsAs(TsAsExpr { ref expr, .. })
         | Expr::TsTypeAssertion(TsTypeAssertion { ref expr, .. })
-        | Expr::TsConstAssertion(TsConstAssertion { ref expr, .. }) => can_be_null(expr),
+        | Expr::TsConstAssertion(TsConstAssertion { ref expr, .. })
+        | Expr::TsInstantiation(TsExprWithTypeArgs { ref expr, .. }) => can_be_null(expr),
         Expr::OptChain(ref e) => can_be_null(&e.expr),
 
         Expr::Invalid(..) => unreachable!(),
