@@ -1,13 +1,17 @@
 #![allow(dead_code)]
 
-use swc_common::{collections::AHashSet, util::take::Take, Mark, Spanned, SyntaxContext, DUMMY_SP};
+use swc_common::{
+    collections::{AHashMap, AHashSet},
+    util::take::Take,
+    Mark, Spanned, SyntaxContext, DUMMY_SP,
+};
 use swc_ecma_ast::*;
 use swc_ecma_transforms_base::{helper, perf::Check};
 use swc_ecma_transforms_classes::super_field::SuperFieldAccessFolder;
 use swc_ecma_transforms_macros::fast_path;
 use swc_ecma_utils::{
     alias_ident_for, alias_if_required, constructor::inject_after_super, default_constructor,
-    private_ident, quote_ident, undefined, ExprFactory, ModuleItemLike, StmtLike,
+    private_ident, quote_ident, undefined, ExprFactory, ModuleItemLike, StmtLike, HANDLER,
 };
 use swc_ecma_visit::{
     as_folder, noop_visit_mut_type, noop_visit_type, Fold, Visit, VisitMut, VisitMutWith, VisitWith,
@@ -15,8 +19,11 @@ use swc_ecma_visit::{
 
 use self::{
     class_name_tdz::ClassNameTdzFolder,
-    private_field::{BrandCheckHandler, FieldAccessFolder, Private, PrivateKind, PrivateRecord},
-    this_in_static::ThisInStaticFolder,
+    private_field::{
+        dup_private_method, visit_private_in_expr, BrandCheckHandler, Private,
+        PrivateAccessVisitor, PrivateKind, PrivateRecord,
+    },
+    this_in_static::{NewTargetInProp, ThisInStaticFolder},
     used_name::UsedNameCollector,
 };
 
@@ -310,29 +317,64 @@ impl ClassProperties {
         let private = Private {
             mark: Mark::fresh(Mark::root()),
             class_name: class_ident.clone(),
-            ident: class
-                .body
-                .iter()
-                .filter_map(|member| match member {
-                    ClassMember::PrivateMethod(method) => Some((
-                        method.key.id.sym.clone(),
-                        PrivateKind {
-                            is_method: true,
-                            is_static: method.is_static,
-                        },
-                    )),
+            ident: {
+                let mut private_map = AHashMap::default();
 
-                    ClassMember::PrivateProp(prop) => Some((
-                        prop.key.id.sym.clone(),
-                        PrivateKind {
-                            is_method: false,
-                            is_static: prop.is_static,
-                        },
-                    )),
+                for member in class.body.iter() {
+                    match member {
+                        ClassMember::PrivateMethod(method) => {
+                            if let Some(kind) = private_map.get_mut(&method.key.id.sym) {
+                                if dup_private_method(kind, method) {
+                                    let error =
+                                        format!("duplicate private name #{}.", method.key.id.sym);
+                                    HANDLER.with(|handler| {
+                                        handler.struct_span_err(method.key.id.span, &error).emit()
+                                    });
+                                } else {
+                                    match method.kind {
+                                        MethodKind::Getter => kind.has_getter = true,
+                                        MethodKind::Setter => kind.has_setter = true,
+                                        MethodKind::Method => unreachable!(),
+                                    }
+                                }
+                            } else {
+                                private_map.insert(
+                                    method.key.id.sym.clone(),
+                                    PrivateKind {
+                                        is_method: true,
+                                        is_static: method.is_static,
+                                        has_getter: method.kind == MethodKind::Getter,
+                                        has_setter: method.kind == MethodKind::Setter,
+                                    },
+                                );
+                            }
+                        }
 
-                    _ => None,
-                })
-                .collect(),
+                        ClassMember::PrivateProp(prop) => {
+                            if private_map.get(&prop.key.id.sym).is_some() {
+                                let error = format!("duplicate private name #{}.", prop.key.id.sym);
+                                HANDLER.with(|handler| {
+                                    handler.struct_span_err(prop.key.id.span, &error).emit()
+                                });
+                            } else {
+                                private_map.insert(
+                                    prop.key.id.sym.clone(),
+                                    PrivateKind {
+                                        is_method: false,
+                                        is_static: prop.is_static,
+                                        has_getter: false,
+                                        has_setter: false,
+                                    },
+                                );
+                            };
+                        }
+
+                        _ => (),
+                    };
+                }
+
+                private_map
+            },
         };
 
         self.private.push(private);
@@ -370,6 +412,8 @@ impl ClassProperties {
                             span: c_span,
                             mut expr,
                         }) => {
+                            vars.extend(visit_private_in_expr(&mut expr, &self.private));
+
                             expr.visit_mut_with(&mut ClassNameTdzFolder {
                                 class_name: &class_ident,
                             });
@@ -410,20 +454,21 @@ impl ClassProperties {
                     }
 
                     let key = match prop.key {
-                        PropName::Ident(ref i) => Box::new(Expr::from(Lit::Str(Str {
+                        PropName::Ident(i) => Box::new(Expr::from(Lit::Str(Str {
                             span: i.span,
-                            value: i.sym.clone(),
+                            value: i.sym,
                             has_escape: false,
                             kind: StrKind::Normal {
                                 contains_quote: false,
                             },
                         }))),
-                        PropName::Num(ref num) => Box::new(Expr::from(*num)),
-                        PropName::Str(ref str) => Box::new(Expr::from(str.clone())),
-                        PropName::BigInt(ref big_int) => Box::new(Expr::from(big_int.clone())),
+                        PropName::Num(num) => Box::new(Expr::from(num)),
+                        PropName::Str(str) => Box::new(Expr::from(str)),
+                        PropName::BigInt(big_int) => Box::new(Expr::from(big_int)),
 
-                        PropName::Computed(key) => {
-                            let (ident, aliased) = if let Expr::Ident(ref i) = *key.expr {
+                        PropName::Computed(mut key) => {
+                            vars.extend(visit_private_in_expr(&mut key.expr, &self.private));
+                            let (ident, aliased) = if let Expr::Ident(i) = &*key.expr {
                                 if used_key_names.contains(&i.sym) {
                                     (alias_ident_for(&key.expr, "_ref"), true)
                                 } else {
@@ -447,6 +492,11 @@ impl ClassProperties {
                     };
 
                     let mut value = prop.value.unwrap_or_else(|| undefined(prop_span));
+
+                    value.visit_mut_with(&mut NewTargetInProp);
+
+                    vars.extend(visit_private_in_expr(&mut value, &self.private));
+
                     if prop.is_static {
                         value.visit_mut_with(&mut SuperFieldAccessFolder {
                             class_name: &class_ident,
@@ -503,6 +553,12 @@ impl ClassProperties {
                         // We use `self.mark` for private variables.
                         prop.key.span.apply_mark(self.private.curr_mark()),
                     );
+
+                    if let Some(value) = &mut prop.value {
+                        value.visit_mut_with(&mut NewTargetInProp);
+                        vars.extend(visit_private_in_expr(&mut *value, &self.private));
+                    }
+
                     prop.value.visit_with(&mut UsedNameCollector {
                         used_names: &mut used_names,
                     });
@@ -708,7 +764,7 @@ impl ClassProperties {
             members.push(ClassMember::Constructor(c));
         }
 
-        private_method_fn_decls.visit_mut_with(&mut FieldAccessFolder {
+        private_method_fn_decls.visit_mut_with(&mut PrivateAccessVisitor {
             private: &self.private,
             vars: vec![],
             in_assign_pat: false,
@@ -716,7 +772,7 @@ impl ClassProperties {
 
         extra_stmts.extend(private_method_fn_decls);
 
-        members.visit_mut_with(&mut FieldAccessFolder {
+        members.visit_mut_with(&mut PrivateAccessVisitor {
             private: &self.private,
             vars: vec![],
             in_assign_pat: false,
