@@ -8,7 +8,9 @@ use swc_common::{
 };
 use swc_ecma_ast::*;
 use swc_ecma_transforms_base::helper;
-use swc_ecma_utils::{alias_ident_for, alias_if_required, prepend, quote_ident, ExprFactory};
+use swc_ecma_utils::{
+    alias_ident_for, alias_if_required, prepend, quote_ident, ExprFactory, HANDLER,
+};
 use swc_ecma_visit::{noop_visit_mut_type, VisitMut, VisitMutWith};
 
 pub(super) struct Private {
@@ -40,21 +42,25 @@ impl PrivateRecord {
         self.0.pop();
     }
 
-    pub fn get(&self, name: &JsWord) -> (Mark, PrivateKind, &Ident) {
+    pub fn get(&self, name: &Ident) -> (Mark, PrivateKind, &Ident) {
         for p in self.0.iter().rev() {
-            if let Some(kind) = p.ident.get(name) {
-                return (p.mark, kind.clone(), &p.class_name);
+            if let Some(kind) = p.ident.get(&name.sym) {
+                return (p.mark, *kind, &p.class_name);
             }
         }
-        // TODO: better error information with span
-        panic!("Private name #{} is not defined.", name);
+
+        let error = format!("private name #{} is not defined.", name.sym);
+        HANDLER.with(|handler| handler.struct_span_err(name.span, &error).emit());
+        (Mark::root(), PrivateKind::default(), &self.0[0].class_name)
     }
 }
 
-#[derive(Clone, PartialEq)]
+#[derive(Copy, Clone, PartialEq, Default)]
 pub(super) struct PrivateKind {
     pub is_static: bool,
     pub is_method: bool,
+    pub has_getter: bool,
+    pub has_setter: bool,
 }
 
 pub(super) struct BrandCheckHandler<'a> {
@@ -93,7 +99,11 @@ impl VisitMut for BrandCheckHandler<'_> {
 
                 self.names.insert(n.id.sym.clone());
 
-                let (mark, kind, class_name) = self.private.get(&n.id.sym);
+                let (mark, kind, class_name) = self.private.get(&n.id);
+
+                if mark == Mark::root() {
+                    return;
+                }
 
                 if kind.is_static {
                     *e = Expr::Bin(BinExpr {
@@ -121,7 +131,7 @@ impl VisitMut for BrandCheckHandler<'_> {
     }
 }
 
-pub(super) struct FieldAccessFolder<'a> {
+pub(super) struct PrivateAccessVisitor<'a> {
     pub vars: Vec<VarDeclarator>,
     pub private: &'a PrivateRecord,
     pub in_assign_pat: bool,
@@ -157,7 +167,7 @@ macro_rules! take_vars {
 }
 
 // super.#sdsa is invalid
-impl<'a> VisitMut for FieldAccessFolder<'a> {
+impl<'a> VisitMut for PrivateAccessVisitor<'a> {
     noop_visit_mut_type!();
 
     take_vars!(visit_mut_function, Function);
@@ -191,7 +201,11 @@ impl<'a> VisitMut for FieldAccessFolder<'a> {
 
                 let obj = arg.obj.clone();
 
-                let (mark, kind, class_name) = self.private.get(&n.id.sym);
+                let (mark, kind, class_name) = self.private.get(&n.id);
+                if mark == Mark::root() {
+                    return;
+                }
+
                 let ident = Ident::new(format!("_{}", n.id.sym).into(), n.id.span.apply_mark(mark));
 
                 let var = alias_ident_for(&obj, "_ref");
@@ -322,7 +336,11 @@ impl<'a> VisitMut for FieldAccessFolder<'a> {
 
                 let obj = left.obj.clone();
 
-                let (mark, kind, class_name) = self.private.get(&n.id.sym);
+                let (mark, kind, class_name) = self.private.get(&n.id);
+                if mark == Mark::root() {
+                    return;
+                }
+
                 let ident = Ident::new(format!("_{}", n.id.sym).into(), n.id.span.apply_mark(mark));
 
                 let var = alias_ident_for(&obj, "_ref");
@@ -486,7 +504,22 @@ impl<'a> VisitMut for FieldAccessFolder<'a> {
     }
 }
 
-impl<'a> FieldAccessFolder<'a> {
+pub(super) fn visit_private_in_expr(
+    expr: &mut Expr,
+    private: &PrivateRecord,
+) -> Vec<VarDeclarator> {
+    let mut priv_visitor = PrivateAccessVisitor {
+        private,
+        vars: vec![],
+        in_assign_pat: false,
+    };
+
+    expr.visit_mut_with(&mut priv_visitor);
+
+    priv_visitor.vars
+}
+
+impl<'a> PrivateAccessVisitor<'a> {
     /// Returns `(expr, thisObject)`
     ///
     ///   - `obj_alias`: If alias is already declared, this method will use
@@ -505,7 +538,11 @@ impl<'a> FieldAccessFolder<'a> {
 
         let mut obj = e.obj.take();
 
-        let (mark, kind, class_name) = self.private.get(&n.id.sym);
+        let (mark, kind, class_name) = self.private.get(&n.id);
+        if mark == Mark::root() {
+            return (Expr::dummy(), None);
+        }
+
         let method_name = Ident::new(
             n.id.sym.clone(),
             n.id.span.with_ctxt(SyntaxContext::empty()).apply_mark(mark),
@@ -650,3 +687,15 @@ impl<'a> FieldAccessFolder<'a> {
         }
     }
 }
+
+/// only getter and setter in same scope could coexist
+pub(super) fn dup_private_method(kind: &PrivateKind, method: &PrivateMethod) -> bool {
+    if !kind.is_method || kind.is_static != method.is_static || method.kind == MethodKind::Method {
+        return true;
+    }
+
+    !matches!(
+        (method.kind, kind.has_getter, kind.has_setter),
+        (MethodKind::Getter, false, true) | (MethodKind::Setter, true, false)
+    )
+}
